package com.zhongshu.reward.server.core.util;

import org.springframework.stereotype.Component;
import redis.clients.jedis.Jedis;

/**
 * @author wjf
 * @date 2024/8/19
 */

public class RedisDistributedLock {
    private Jedis jedis;
    private String lockKey;
    private int expireTime; // 锁的超时时间

    public RedisDistributedLock(Jedis jedis, String lockKey, int expireTime) {
        this.jedis = jedis;
        this.lockKey = lockKey;
        this.expireTime = expireTime;
    }

    public boolean lock() {
        long expires = System.currentTimeMillis() + expireTime + 1;
        String expiresStr = String.valueOf(expires); // 锁到期时间

        if (jedis.setnx(lockKey, expiresStr) == 1) {
            // 获取锁成功
            jedis.expire(lockKey, expireTime); // 设置锁的过期时间
            return true;
        }

        String currentValueStr = jedis.get(lockKey); // Redis里面的时间
        if (currentValueStr != null && Long.parseLong(currentValueStr) < System.currentTimeMillis()) {
            // 锁已经存在，但已经过期，可以尝试获取锁
            String oldValueStr = jedis.getSet(lockKey, expiresStr);
            if (oldValueStr != null && oldValueStr.equals(currentValueStr)) {
                // 获取锁成功
                jedis.expire(lockKey, expireTime); // 设置锁的过期时间
                return true;
            }
        }
        // 其他情况，无法获取锁
        return false;
    }

    public void unlock() {
        // 确保是该用户持有的锁才能释放
        String currentValueStr = jedis.get(lockKey);
        if (currentValueStr != null && Long.parseLong(currentValueStr) == System.currentTimeMillis()) {
            jedis.del(lockKey); // 释放锁
        }
    }

    public static void main(String[] args) {
        Jedis jedis = new Jedis("localhost");
        RedisDistributedLock lock = new RedisDistributedLock(jedis, "user_1001", 1000);

        try {
            if (lock.lock()) {
                // 执行业务逻辑
                System.out.println("执行业务逻辑...");
                // 模拟耗时操作
                Thread.sleep(2000);
            } else {
                System.out.println("未获取到锁，其他实例可能正在操作...");
            }
        } catch (InterruptedException e) {
            e.printStackTrace();
        } finally {
            lock.unlock();
            jedis.close();
        }
    }
}
